#define TERMINAL "alacritty"
#define TERMCLASS "alacritty"
#define BROWSER "librewolf"

/* appearance */
static const int sloppyfocus               = 1;  /* focus follows mouse */
static const int bypass_surface_visibility = 0;  /* 1 means idle inhibitors will disable idle tracking even if it's surface isn't visible  */
static const unsigned int borderpx         = 3;  /* border pixel of windows */
static const float bordercolor[]           = {0.3, 0.3, 0.3, 1.0};
static const float focuscolor[]            = {0.92, 0.51, 0.91, 1.0};
/* To conform the xdg-protocol, set the alpha to zero to restore the old behavior */
static const float fullscreen_bg[]         = {0.1, 0.1, 0.1, 1.0};

/* tagging - tagcount must be no greater than 31 */
static const int tagcount = 9;

static const Rule rules[] = {
  /* soft_id    title       tags mask     isfloating   monitor */
  { "Gimp",     NULL,       1 << 8,       0,           -1 },
};

/* layout(s) */
static const Layout layouts[] = {
  /* symbol     arrange function */
  { "[]=",      tile },
  { "><>",      NULL },    /* no layout function means floating behavior */
  { "[M]",      monocle },
};

/* autostart */
static const char *const autostart[] = {
  "wbg", "/home/suwako/.local/share/wallpapers/hurandorusukaretsuto-kabegami-2560x1600_7-1080.jpg", NULL,
  "dwl-bar", NULL,
  "someblocks", NULL,
  NULL /* terminate */
};

/* monitors */
static const MonitorRule monrules[] = {
  /* name       mfact nmaster scale layout       rotate/reflect                x    y */
  /* example of a HiDPI laptop monitor:
  { "eDP-1",    0.5,  1,      2,    &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL,   -1,  -1 },
  */
  /* defaults */
  { NULL,       0.55, 1,      1,    &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL,   -1,  -1 },
};

/* keyboard */
static const struct xkb_rule_names xkb_rules = {
  /* can specify fields: rules, model, layout, variant, options */
  /* example:
  .options = "ctrl:nocaps",
  */
  .layout = "jp",
};

static const int repeat_rate = 25;
static const int repeat_delay = 600;

/* Trackpad */
static const int tap_to_click = 1;
static const int tap_and_drag = 1;
static const int drag_lock = 1;
static const int natural_scrolling = 0;
static const int disable_while_typing = 1;
static const int left_handed = 0;
static const int middle_button_emulation = 0;
/* You can choose between:
LIBINPUT_CONFIG_SCROLL_NO_SCROLL
LIBINPUT_CONFIG_SCROLL_2FG
LIBINPUT_CONFIG_SCROLL_EDGE
LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN
*/
static const enum libinput_config_scroll_method scroll_method = LIBINPUT_CONFIG_SCROLL_2FG;

/* You can choose between:
LIBINPUT_CONFIG_CLICK_METHOD_NONE
LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS
LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER
*/
static const enum libinput_config_click_method click_method = LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;

/* You can choose between:
LIBINPUT_CONFIG_SEND_EVENTS_ENABLED
LIBINPUT_CONFIG_SEND_EVENTS_DISABLED
LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE
*/
static const uint32_t send_events_mode = LIBINPUT_CONFIG_SEND_EVENTS_ENABLED;

/* You can choose between:
LIBINPUT_CONFIG_ACCEL_PROFILE_FLAT
LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE
*/
static const enum libinput_config_accel_profile accel_profile = LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE;
static const double accel_speed = 0.0;
/* You can choose between:
LIBINPUT_CONFIG_TAP_MAP_LRM -- 1/2/3 finger tap maps to left/right/middle
LIBINPUT_CONFIG_TAP_MAP_LMR -- 1/2/3 finger tap maps to left/middle/right
*/
static const enum libinput_config_tap_button_map button_map = LIBINPUT_CONFIG_TAP_MAP_LRM;

/* If you want to use the windows key for MODKEY, use WLR_MODIFIER_LOGO */
#define MODKEY WLR_MODIFIER_LOGO

#define TAGKEYS(KEY,TAG) \
  { MODKEY,                    KEY,            view,            {.ui = 1 << TAG} }, \
  { MODKEY|WLR_MODIFIER_CTRL,  KEY,            toggleview,      {.ui = 1 << TAG} }, \
  { MODKEY|WLR_MODIFIER_SHIFT, KEY,            tag,             {.ui = 1 << TAG} }, \
  { MODKEY|WLR_MODIFIER_CTRL|WLR_MODIFIER_SHIFT,KEY,toggletag,  {.ui = 1 << TAG} }
#define STACKKEYS(MOD,ACTION) \
  { MOD,  XK_j, ACTION##stack,  {.i = INC(+1) } }, \
  { MOD,  XK_k, ACTION##stack,  {.i = INC(-1) } }, \
  { MOD,  XK_v,   ACTION##stack,  {.i = 0 } }, \
  /* { MOD, XK_grave, ACTION##stack, {.i = PREVSEL } }, \ */
  /* { MOD, XK_a,     ACTION##stack, {.i = 1 } }, \ */
  /* { MOD, XK_z,     ACTION##stack, {.i = 2 } }, \ */
  /* { MOD, XK_x,     ACTION##stack, {.i = -1 } }, */

/* helper for spawning shell commands in the pre dwm-5.0 fashion */
#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }

/* commands */
static const char *termcmd[] = { TERMINAL, NULL };

#include "shiftview.c"

static const Key keys[] = {
  /* Note that Shift changes certain key codes: c -> C, 2 -> at, etc. */
  /* modifier                  key                  function        argument */
  TAGKEYS(                     XKB_KEY_1,           0),
  TAGKEYS(                     XKB_KEY_2,           1),
  TAGKEYS(                     XKB_KEY_3,           2),
  TAGKEYS(                     XKB_KEY_4,           3),
  TAGKEYS(                     XKB_KEY_5,           4),
  TAGKEYS(                     XKB_KEY_6,           5),
  TAGKEYS(                     XKB_KEY_7,           6),
  TAGKEYS(                     XKB_KEY_8,           7),
  TAGKEYS(                     XKB_KEY_9,           8),

  { MODKEY,                    XKB_KEY_0,           view,           {.ui = ~0} },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_0,           tag,            {.ui = ~0} },

  { MODKEY,                    XKB_KEY_Tab,         view,           {0} },
  { MODKEY,                    XKB_KEY_q,           killclient,     {0} },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_Q,           quit,           {0} },
  { MODKEY,                    XKB_KEY_t,           setlayout,      {.v = &layouts[0]} },
  { MODKEY,                    XKB_KEY_i,           incnmaster,     {.i = +1} },
  { MODKEY,                    XKB_KEY_0,           view,           {.ui = ~0} },
  { MODKEY,                    XKB_KEY_p,           spawn,          {.v = (const char*[]){ "mpc", "toggle", NULL } } },
  //{ MODKEY,                    XKB_KEY_Return,      zoom,           {0} },
  { MODKEY,                    XKB_KEY_Return,      spawn,          {.v = termcmd} },

  { MODKEY,                    XKB_KEY_Eisu_toggle, spawn,          {.v = (const char*[]){ "wofi-calc", NULL } } },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_S,           spawn,          SHCMD("swaylock -i ~/.local/share/wallpapers/hurandorusukaretsuto-kabegami-2560x1600_7-1080.png") },
  { MODKEY,                    XKB_KEY_d,           spawn,          {.v = (const char*[]){ "wofi", "--show", "drun", NULL } } },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_D,           spawn,          {.v = (const char*[]){ "wofi-pass", NULL } } },
  { MODKEY,                    XKB_KEY_f,           togglefullscreen, {0} },
  //{ MODKEY,                    XKB_KEY_f,           setlayout,      {.v = &layouts[1]} },
  //{ MODKEY,                    XKB_KEY_g,           shiftview,      { .i = -1 } },
  //{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_G,           shifttag,       { .i = -1 } },
  { MODKEY,                    XKB_KEY_h,           setmfact,       {.f = -0.05} },
  { MODKEY,                    XKB_KEY_j,           focusstack,     {.i = +1} },
  { MODKEY,                    XKB_KEY_k,           focusstack,     {.i = -1} },
  { MODKEY,                    XKB_KEY_l,           setmfact,       {.f = +0.05} },

  { MODKEY,                    XKB_KEY_z,           spawn,          {.v = (const char*[]){ "wofi-youtube", "-i", NULL } } },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_Z,           spawn,          {.v = (const char*[]){ "wofi-youtube", "-s", NULL } } },
  { MODKEY,                    XKB_KEY_b,           toggle_visibility, {0}},
  { MODKEY,                    XKB_KEY_m,           setlayout,      {.v = &layouts[2]} },
  //{ MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_parenright, tag,            {.ui = ~0} },

  { MODKEY,                    XKB_KEY_comma,       spawn,          {.v = (const char*[]){ "mpc", "prev", NULL } } },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_less,        spawn,          {.v = (const char*[]){ "mpc", "seek", "0%", NULL } } },
  { MODKEY,                    XKB_KEY_period,      spawn,          {.v = (const char*[]){ "mpc", "next", NULL } } },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_greater,     spawn,          {.v = (const char*[]){ "mpc", "repeat", NULL } } },

  { MODKEY,                    XKB_KEY_space,       zoom,           {0} },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_space,       togglefloating, {0} },

  { MODKEY,                    XKB_KEY_Left,        focusmon,       {.i = WLR_DIRECTION_LEFT} },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_Left,        tagmon,         {.i = WLR_DIRECTION_LEFT} },
  { MODKEY,                    XKB_KEY_Right,       focusmon,       {.i = WLR_DIRECTION_RIGHT} },
  { MODKEY|WLR_MODIFIER_SHIFT, XKB_KEY_Right,       tagmon,         {.i = WLR_DIRECTION_RIGHT} },

  /* Ctrl-Alt-Backspace and Ctrl-Alt-Fx used to be handled by X server */
  { WLR_MODIFIER_CTRL|WLR_MODIFIER_ALT,XKB_KEY_Terminate_Server, quit, {0} },
#define CHVT(n) { WLR_MODIFIER_CTRL|WLR_MODIFIER_ALT,XKB_KEY_XF86Switch_VT_##n, chvt, {.ui = (n)} }
  CHVT(1), CHVT(2), CHVT(3), CHVT(4), CHVT(5), CHVT(6),
  CHVT(7), CHVT(8), CHVT(9), CHVT(10), CHVT(11), CHVT(12),
};

static const Button buttons[] = {
  { MODKEY, BTN_LEFT,   moveresize,     {.ui = CurMove} },
  { MODKEY, BTN_MIDDLE, togglefloating, {0} },
  { MODKEY, BTN_RIGHT,  moveresize,     {.ui = CurResize} },
};
